@@ -10,37 +10,56 @@ from contextlib import contextmanager
 
 
 __all__ = ['Dispatcher', 'command', 'dispatch']
-__version__ = '3.8'
+__version__ = '4.0'
 __author__ = 'Alexander Solovyov'
 __email__ = 'alexander@solovyov.net'
 
 
-try:
-    import locale
-    ENCODING = locale.getpreferredencoding()
-    if (not ENCODING or ENCODING == 'mac-roman' or 'ascii' in ENCODING.lower()
-        or 'ansi' in ENCODING.lower() or ENCODING.lower() == 'cp1252'):
-        ENCODING = 'UTF-8'
-except locale.Error:
-    ENCODING = 'UTF-8'
-
-_writer = codecs.getwriter(ENCODING)
-
-
 def write(text, out=None):
     '''Write output to a given stream (stdout by default).'''
     out = out or sys.stdout
-    if hasattr(out, 'buffer'):
-        out = _writer(out.buffer)
-    elif sys.version_info < (3, 0) and isinstance(text, unicode):
-        text = text.encode(ENCODING)
-    out.write(text)
+    try:
+        print >> out, text
+    # Needed on Python 2.x if text is str/bytes containing non-ascii
+    # characters and sys.stdout is replaced by a writer from the codecs
+    # module. text will be decoded as ascii giving the decode error.
+    except UnicodeDecodeError:
+        print >> out, text.decode('utf-8')
+    # Get the order of stdout/stderr correct on Windows. AFAICT this is only
+    # needed for the test environment but it's harmless otherwise.
+    out.flush()
 
 
 def err(text):
     '''Write output to stderr.'''
     write(text, out=sys.stderr)
-    sys.stderr.flush()
+
+
+# encoding to use when decoding command line arguments
+FSE_ENCODING = sys.getfilesystemencoding()
+ARG_ENCODING = os.environ.get('OPSTER_ARG_ENCODING', FSE_ENCODING)
+
+
+def decodearg(arg, arg_encoding=ARG_ENCODING):
+    '''Decode an argument from sys.argv'''
+    # python 2.x: have bytes, convert to unicode with given encoding
+    if sys.version_info < (3, 0):
+        return arg.decode(arg_encoding)
+
+    # python 3.x: have unicode
+    # arg has already been decoded with FSE_ENCODING
+    # In the default case we just return the arg as it is
+    if arg_encoding == FSE_ENCODING:
+        return arg
+
+    # Need to encode and redecode as arg_encoding
+    if os.name == 'posix':
+        # On posix the argument was decoded using surrogate escape
+        arg = arg.encode(FSE_ENCODING, 'surrogateescape')
+    else:
+        # On windows the 'mbcs' codec has no surrogate escape handler
+        arg = arg.encode(FSE_ENCODING)
+    return arg.decode(arg_encoding)
 
 
 class Dispatcher(object):
@@ -54,7 +73,7 @@ class Dispatcher(object):
     '''
 
     def __init__(self, cmdtable=None, globaloptions=None, middleware=None):
-        self._cmdtable = cmdtable or {}
+        self._cmdtable = CmdTable(cmdtable or {})
         self._globaloptions = [Option(o) for o in (globaloptions or [])]
         self.middleware = middleware
 
@@ -67,9 +86,7 @@ class Dispatcher(object):
 
     @property
     def cmdtable(self):
-        table = self._cmdtable.copy()
-        table['help'] = help_(table, self.globaloptions), [], '[TOPIC]'
-        return table
+        return self._cmdtable.copy()
 
     def command(self, options=None, usage=None, name=None, shortlist=False,
                 hide=False, aliases=()):
@@ -129,7 +146,7 @@ class Dispatcher(object):
                 options_ = []
 
             cmdname = name or name_from_python(func.__name__)
-            scriptname = name or sysname()
+            scriptname_ = name or sysname()
             if usage is None:
                 usage_ = guess_usage(func, options_)
             else:
@@ -140,33 +157,27 @@ class Dispatcher(object):
                 cmdname = cmdname + '|' + '|'.join(aliases)
             self._cmdtable[cmdname] = (func, options_, usage_)
 
-            def help_func(name=None):
-                return help_cmd(func, replace_name(usage_, sysname()), options_,
-                                aliases)
+            def help_func(scriptname=None):
+                scriptname = scriptname or sysname()
+                return help_cmd(func, usage_, options_, aliases, scriptname)
 
-            def command(argv=None):
-                for o in self.globaloptions:
-                    # Don't include global option if long name matches
-                    if any((x.name == o.name for x in options_)):
-                        continue
-                    # Don't use global option short name if already used
-                    if any((x.short and x.short == o.short
-                            for x in options_)):
-                        o = o._replace(short='')
-                    options_.append(o)
+            def command(argv=None, scriptname=None):
+                scriptname = scriptname or sysname()
+                merge_globalopts(self.globaloptions, options_)
 
                 if argv is None:
                     argv = sys.argv[1:]
 
                 try:
-                    with exchandle(func.help):
+                    with exchandle(func.help, scriptname):
                         args, opts = process(argv, options_)
 
                     if opts.pop('help', False):
-                        return func.help()
+                        return func.help(scriptname)
 
-                    with exchandle(func.help):
-                        return call_cmd(scriptname, func, options_)(*args, **opts)
+                    with exchandle(func.help, scriptname):
+                        with help_workaround(func, scriptname):
+                            return call_cmd(scriptname, func, options_)(*args, **opts)
 
                 except ErrorHandled:
                     return -1
@@ -176,30 +187,49 @@ class Dispatcher(object):
             func.command = command
             func.opts = options_
             func.orig = func
+            func.scriptname = scriptname_
 
             @wraps(func)
             def inner(*args, **opts):
                 return call_cmd_regular(func, options_)(*args, **opts)
 
+            # Store this for help_workaround
+            func._inner = inner
+
             return inner
 
         return wrapper
 
-    def dispatch(self, args=None):
+    def nest(self, name, dispatcher, help, hide=False, shortlist=False):
+        '''Add another dispatcher as a subcommand.'''
+        dispatcher.__doc__ = help
+        prefix = hide and '~' or (shortlist and '^' or '')
+        self._cmdtable[prefix + name] = dispatcher, [], None
+
+    def dispatch(self, args=None, scriptname=None):
         '''Dispatch command line arguments using subcommands.
 
         - ``args``: list of arguments, default: ``sys.argv[1:]``
         '''
-        args = args or sys.argv[1:]
+        if args is None:
+            args = sys.argv[1:]
+        scriptname = scriptname or sysname()
 
-        help_func = self.cmdtable['help'][0]
-        autocomplete(self.cmdtable, args, self.middleware)
+        # Add help function to the table
+        cmdtable = self.cmdtable
+        help_func = help_(cmdtable, self.globaloptions, scriptname)
+        cmdtable['help'] = help_func, [], '[TOPIC]'
+
+        autocomplete(cmdtable, args, self.middleware)
 
         try:
             with exchandle(help_func):
-                cmd, func, args, options = cmdparse(args, self.cmdtable,
+                cmd, func, args, options = cmdparse(args, cmdtable,
                                                     self.globaloptions)
 
+            if isinstance(func, Dispatcher):
+                return func.dispatch(args, scriptname=scriptname + ' ' + cmd)
+
             with exchandle(help_func, cmd):
                 args, opts = process(args, options)
 
@@ -210,7 +240,8 @@ class Dispatcher(object):
 
             mw = cmd != '_completion' and self.middleware or None
             with exchandle(help_func, cmd):
-                return call_cmd(cmd, func, options, mw)(*args, **opts)
+                with help_workaround(func, cmd, help_func):
+                    return call_cmd(cmd, func, options, mw)(*args, **opts)
 
         except ErrorHandled:
             return -1
@@ -229,18 +260,19 @@ def command(options=None, usage=None, name=None, shortlist=False, hide=False,
 command.__doc__ = Dispatcher.command.__doc__
 
 
-def dispatch(args=None, cmdtable=None, globaloptions=None, middleware=None):
+def dispatch(args=None, cmdtable=None, globaloptions=None, middleware=None,
+             scriptname=None):
     global _dispatcher
     if not _dispatcher:
         _dispatcher = Dispatcher(cmdtable, globaloptions, middleware)
     else:
         if cmdtable:
-            _dispatcher._cmdtable = cmdtable
+            _dispatcher._cmdtable = CmdTable(cmdtable)
         if globaloptions:
             _dispatcher._globaloptions = [Option(o) for o in globaloptions]
         if middleware:
             _dispatcher.middleware = middleware
-    return _dispatcher.dispatch(args)
+    return _dispatcher.dispatch(args, scriptname)
 dispatch.__doc__ = Dispatcher.dispatch.__doc__
 
 
@@ -248,10 +280,10 @@ dispatch.__doc__ = Dispatcher.dispatch.__doc__
 # Help
 # --------
 
-def help_(cmdtable, globalopts):
+def help_(cmdtable, globalopts, scriptname):
     '''Help generator for a command table.
     '''
-    def help_inner(name=None, **opts):
+    def help_inner(name=None, *args, **opts):
         '''Show help for a given help topic or a help overview.
 
         With no arguments, print a list of commands with short help messages.
@@ -276,27 +308,34 @@ def help_(cmdtable, globalopts):
             hlplist = sorted(hlp)
             maxlen = max(map(len, hlplist))
 
-            write('usage: %s <command> [options]\n' % sysname())
-            write('\ncommands:\n\n')
+            write('usage: %s <command> [options]' % scriptname)
+            write('\ncommands:\n')
             for cmd in hlplist:
                 doc = hlp[cmd]
-                write(' %-*s  %s\n' % (maxlen, cmd.split('|', 1)[0], doc))
+                write(' %-*s  %s' % (maxlen, cmd.split('|', 1)[0], doc))
 
         if not cmdtable:
-            return err('No commands specified!\n')
+            return err('No commands specified!')
 
         if not name or name == 'shortlist':
             return helplist()
 
         aliases, (cmd, options, usage) = findcmd(name, cmdtable)
-        return help_cmd(cmd,
-                        replace_name(usage, sysname() + ' ' + aliases[0]),
-                        options + globalopts,
-                        aliases[1:])
+
+        if isinstance(cmd, Dispatcher):
+            recurse = help_(cmd.cmdtable, globalopts, scriptname + ' ' + name)
+            return recurse(*args, **opts)
+
+        options = list(options)
+        merge_globalopts(globalopts, options)
+
+        return help_cmd(cmd, usage, options, aliases[1:],
+                        scriptname + ' ' + aliases[0])
+
     return help_inner
 
 
-def help_cmd(func, usage, options, aliases):
+def help_cmd(func, usage, options, aliases, scriptname=None):
     '''Show help for given command.
 
     - ``func``: function to generate help for (``func.__doc__`` is taken)
@@ -316,7 +355,7 @@ def help_cmd(func, usage, options, aliases):
     ...          'daemonize process'),
     ...         ('', 'pid-file', '',
     ...          'name of file to write process ID to')]
-    >>> help_cmd(test, 'test [-l HOST] [NAME]', opts, ())
+    >>> help_cmd(test, '%name [-l HOST] [NAME]', opts, (), 'test')
     test [-l HOST] [NAME]
     <BLANKLINE>
     that's a test command
@@ -331,19 +370,20 @@ def help_cmd(func, usage, options, aliases):
         --pid-file   name of file to write process ID to
     '''
     options = [Option(o) for o in options]  # only for doctest
-    write(usage + '\n')
+    usage = replace_name(usage, scriptname)
+    write(usage)
     if aliases:
-        write('\naliases: ' + ', '.join(aliases) + '\n')
+        write('\naliases: ' + ', '.join(aliases))
     doc = pretty_doc_string(func)
-    write('\n' + doc.strip() + '\n\n')
-    if options:
-        write(''.join(help_options(options)))
+    write('\n' + doc.strip() + '\n')
+    for line in help_options(options):
+        write(line)
 
 
 def help_options(options):
     '''Generator for help on options.
     '''
-    yield 'options:\n\n'
+    yield 'options:\n'
     output = []
     for o in options:
         default = o.default_value()
@@ -358,15 +398,26 @@ def help_options(options):
             # wrap description at 78 chars
             second = textwrap.wrap(second, width=(78 - opts_len - 3))
             pad = '\n' + ' ' * (opts_len + 3)
-            yield ' %-*s  %s\n' % (opts_len, first, pad.join(second))
+            yield ' %-*s  %s' % (opts_len, first, pad.join(second))
         else:
-            yield '%s\n' % first
+            yield ' %s' % first
 
 
 # --------
 # Options process
 # --------
 
+def merge_globalopts(globalopts, opts):
+    '''Merge the global options with the subcommand options'''
+    for o in globalopts:
+        # Don't include global option if long name matches
+        if any((x.name == o.name for x in opts)):
+            continue
+        # Don't use global option short name if already used
+        if any((x.short and x.short == o.short for x in opts)):
+            o = o._replace(short='')
+        opts.append(o)
+
 # Factory for creating _Option instances. Intended to be the entry point to
 # the *Option classes here.
 def Option(opt):
@@ -389,12 +440,18 @@ def Option(opt):
 
     # Find matching _Option subclass and return instance
     # nb. the order of testing matters
-    for Type in (BoolOption, IntOption, FloatOption, ListOption,
-                 DictOption, FuncOption, LiteralOption):
+    for Type in (BoolOption, ListOption, DictOption, FuncOption,
+                 TupleOption, UnicodeOption, LiteralOption):
         if Type.matches(default):
             return Type(*args)
     raise OpsterError('Cannot figure out type for option %s' % name)
 
+def CmdTable(cmdtable):
+    '''Factory to convert option tuples in a cmdtable'''
+    newtable = {}
+    for name, (func, opts, usage) in cmdtable.items():
+        newtable[name] = (func, [Option(o) for o in opts], usage)
+    return newtable
 
 # Superclass for all option classes
 class BaseOption(namedtuple('Option', (
@@ -421,7 +478,7 @@ class BaseOption(namedtuple('Option', (
 
     def convert(self, final):
         '''Generate the resulting python value from the final state.'''
-        return self.type(final)
+        return final
 
     def default_value(self):
         '''Shortcut to obtain the default value when option arg not provided.'''
@@ -429,11 +486,23 @@ class BaseOption(namedtuple('Option', (
 
 
 class LiteralOption(BaseOption):
-    '''Literal option type (including string options, no processing).'''
+    '''Literal option type (including string, int, float, etc.)'''
     type = object
 
     def convert(self, final):
-        return final
+        '''Generate the resulting python value from the final state.'''
+        if final is self.default:
+            return final
+        else:
+            return type(self.default)(final)
+
+
+class UnicodeOption(BaseOption):
+    '''Handle unicode values, decoding input'''
+    type = unicode
+
+    def convert(self, final):
+        return decodearg(final)
 
 
 class BoolOption(BaseOption):
@@ -448,16 +517,6 @@ class BoolOption(BaseOption):
         return not self.default
 
 
-class IntOption(BaseOption):
-    '''Integer number option type.'''
-    type = int
-
-
-class FloatOption(BaseOption):
-    '''Floating point number option type.'''
-    type = float
-
-
 class ListOption(BaseOption):
     '''List option type.'''
     type = list
@@ -487,6 +546,28 @@ class DictOption(BaseOption):
         return state
 
 
+class TupleOption(BaseOption):
+    '''Tuple option type.'''
+    type = tuple
+
+    def __init__(self, *args, **kwargs):
+        self._option = Option(('', '_', self.default[0], ''))
+
+    def default_state(self):
+        return self._option.default
+
+    def update_state(self, state, new):
+        return self._option.update_state(state, new)
+
+    def convert(self, final):
+        finalval = self._option.convert(final)
+        if finalval not in self.default:
+            msg = "unrecognised value: %r (should be one of %s)"
+            msg = msg % (final, ', '.join(str(v) for v in self.default))
+            raise getopt.GetoptError(msg)
+        return finalval
+
+
 class FuncOption(BaseOption):
     '''Function option type.'''
     type = Callable
@@ -584,7 +665,8 @@ def cmdparse(args, cmdtable, globalopts):
         args.remove(cmdarg)
         aliases, info = findcmd(cmdarg, cmdtable)
         cmd = aliases[0]
-        possibleopts = list(info[1]) + globalopts
+        possibleopts = list(info[1])
+        merge_globalopts(globalopts, possibleopts)
         return cmd, info[0] or None, args, possibleopts
     else:
         return None, None, args, globalopts
@@ -647,20 +729,43 @@ def guess_options(func):
 
     See docstring of ``command()`` for description of those variables.
     '''
-    args, _, _, defaults = inspect.getargspec(func)
+    try:
+        args, _, _, defaults = inspect.getargspec(func)
+    except ValueError: # has keyword-only arguments
+        for o in guess_options_py3k(func):
+            yield o
+        return
     for name, option in zip(args[-len(defaults):], defaults):
         if not isinstance(option, tuple):
             continue
         yield (option[0], name_from_python(name)) + option[1:]
 
 
+def guess_options_py3k(func):
+    '''Get options definitions from function with keyword-only arguments
+
+    They should be declared in a following way:
+
+    def func(*, longname=(shortname, default, help)):
+        pass
+
+    See docstring of ``command()`` for description of those variables.
+    '''
+    spec = inspect.getfullargspec(func)
+    for name, option in spec.kwonlydefaults.items():
+        yield (option[0], name_from_python(name)) + option[1:]
+
+
 def guess_usage(func, options):
     '''Get usage definition for a function
     '''
     usage = ['%name']
     if options:
         usage.append('[OPTIONS]')
-    arginfo = inspect.getargspec(func)
+    try:
+        arginfo = inspect.getargspec(func)
+    except ValueError: # keyword-only args
+        arginfo = inspect.getfullargspec(func)
     optnames = [o.name for o in options]
     nonoptional = len(arginfo.args) - len(arginfo.defaults or ())
 
@@ -674,6 +779,27 @@ def guess_usage(func, options):
     return ' '.join(usage)
 
 
+@contextmanager
+def help_workaround(func, scriptname, help_func=None):
+    '''Context manager to temporarily replace func.help'''
+    # Retrieve inner if function is command wrapped
+    func = getattr(func, '_inner', func)
+
+    # Ignore function that was not command wrapped
+    if not hasattr(func, 'help'):
+        yield
+        return
+
+    # Wrap the with block with a replaced help function
+    help = func.help
+    help_func = help_func or help
+    try:
+        func.help = lambda: help_func(scriptname)
+        yield
+    finally:
+        func.help = help
+
+
 @contextmanager
 def exchandle(help_func, cmd=None):
     '''Context manager to turn internal exceptions into printed help messages.
@@ -685,18 +811,18 @@ def exchandle(help_func, cmd=None):
         yield  # execute the block in the 'with' statement
         return
     except UnknownCommand as e:
-        err("unknown command: '%s'\n" % e)
+        err("unknown command: '%s'" % e)
     except AmbiguousCommand as e:
-        err("command '%s' is ambiguous:\n    %s\n" %
+        err("command '%s' is ambiguous:\n    %s" %
             (e.args[0], ' '.join(e.args[1])))
     except ParseError as e:
-        err('%s: %s\n\n' % (e.args[0], e.args[1].strip()))
+        err('%s: %s\n' % (e.args[0], e.args[1].strip()))
         help_func(cmd)
     except getopt.GetoptError as e:
-        err('error: %s\n\n' % e)
+        err('error: %s\n' % e)
         help_func(cmd)
     except OpsterError as e:
-        err('%s\n' % e)
+        err('%s' % e)
     # abort if a handled exception was raised
     raise ErrorHandled()
 
@@ -705,7 +831,10 @@ def call_cmd(name, func, opts, middleware=None):
     '''Wrapper for command call, catching situation with insufficient arguments.
     '''
     # depth is necessary when there is a middleware in setup
-    arginfo = inspect.getargspec(func)
+    try:
+        arginfo = inspect.getargspec(func)
+    except ValueError:
+        arginfo = inspect.getfullargspec(func)
     if middleware:
         tocall = middleware(func)
         depth = 2
@@ -742,20 +871,61 @@ def call_cmd(name, func, opts, middleware=None):
 def call_cmd_regular(func, opts):
     '''Wrapper for command for handling function calls from Python.
     '''
-    def inner(*args, **kwargs):
+    # This would raise an error if there were any keyword-only arguments
+    try:
         arginfo = inspect.getargspec(func)
-        if len(args) > len(arginfo.args):
-            raise TypeError('You have supplied more positional arguments'
-                            ' than applicable')
+    except ValueError:
+        return call_cmd_regular_py3k(func, opts)
+
+    def inner(*args, **kwargs):
+        # Map from argument names to Option instances
+        opt_args = dict((o.pyname, o) for o in opts)
+
+        # Pull any recognised args out of kwargs and splice them with the
+        # positional arguments to give a flat positional arg list
+        remaining = list(args)
+        args = []
+        defaults_offset = len(arginfo.args) - len(arginfo.defaults or [])
+        for n, argname in enumerate(arginfo.args):
+            # Option arguments MUST be given as keyword arguments
+            if argname in opt_args:
+                if argname in kwargs:
+                    argval = kwargs.pop(argname)
+                else:
+                    argval = opt_args[argname].default_value()
+            # Take a positional argument
+            elif remaining:
+                argval = remaining.pop(0)
+            # Find the default value of the positional argument
+            elif n >= defaults_offset:
+                argval = arginfo.defaults[n - defaults_offset]
+            else:
+                raise TypeError('Not enough positional arguments')
+            # Accumulate the args in order
+            args.append(argval)
+
+        # Combine the remaining positional arguments that go to varargs
+        args = args + remaining
+
+        # kwargs is any keyword arguments that were not recognised as options
+        return func(*args, **kwargs)
 
-        # short name, long name, default, help, (maybe) completer
-        funckwargs = dict((o.pyname, o.default) for o in opts)
-        if 'help' not in (arginfo.defaults or ()) and not arginfo.keywords:
-            funckwargs.pop('help', None)
-        funckwargs.update(kwargs)
-        return func(*args, **funckwargs)
     return inner
 
+def call_cmd_regular_py3k(func, opts):
+    '''call_cmd_regular for functions with keyword only arguments'''
+    spec = inspect.getfullargspec(func)
+
+    def inner(*args, **kwargs):
+
+        # Replace the option arguments with their default values
+        for o in opts:
+            if o.pyname not in kwargs:
+                kwargs[o.pyname] = o.default_value()
+
+        return func(*args, **kwargs)
+
+    return inner
 
 def replace_name(usage, name):
     '''Replace name placeholder with a command name.'''
